from __future__ import annotations

from typing import TYPE_CHECKING, Any, ClassVar

from django.conf import settings
from django.contrib.admin.widgets import SELECT2_TRANSLATIONS
from django.forms import Media
from django.urls import reverse
from django.utils.translation import get_language

from .mixin import MediaDefinitionFilter, SmartFieldListFilter

if TYPE_CHECKING:
    from django.contrib.admin import ModelAdmin
    from django.contrib.admin.views.main import ChangeList
    from django.db.models import Field, Model
    from django.http import HttpRequest


def get_real_field(model: Model, path: str) -> Field | None:
    parts = path.split("__")
    current = model
    f = None
    for p in parts:
        f = current._meta.get_field(p)
        if f.related_model:
            current = f.related_model
    return f


class AutoCompleteFilter(SmartFieldListFilter, MediaDefinitionFilter):
    template = "adminfilters/autocomplete.html"
    filter_title = None
    parent = None
    parent_lookup_kwarg = None
    ajax_url = None
    css = "adminfilters/adminfilters%s.css"

    def __init__(
        self,
        field: Field,
        request: HttpRequest,
        params: dict[str, str],
        model: Model,
        model_admin: ModelAdmin,
        field_path: str,
    ) -> None:
        self.dependants = []
        self.lookup_kwarg = f"{field_path}__exact"
        self.lookup_kwarg_isnull = f"{field_path}__isnull"
        self._params = params
        self.request = request
        self.lookup_val = self.get_parameters(self.lookup_kwarg)
        super().__init__(field, request, params, model, model_admin, field_path)
        self.admin_site = model_admin.admin_site
        self.query_string = ""
        self.target_field = get_real_field(model, field_path)
        self.target_model = self.target_field.related_model
        self.target_opts = self.target_field.model._meta

    def expected_parameters(self) -> list[str]:
        return [self.lookup_kwarg, self.lookup_kwarg_isnull]

    @property
    def url(self) -> str:
        return self.get_url()

    def get_url(self) -> str:
        if self.ajax_url is None:
            return reverse(f"{self.admin_site.name}:autocomplete")
        return reverse(self.ajax_url)

    def choices(self, changelist: ChangeList) -> list[str]:
        self.query_string = changelist.get_query_string(remove=[self.lookup_kwarg, self.lookup_kwarg_isnull])
        if self.lookup_val:
            get_kwargs = {self.field.target_field.name: self.lookup_val}
            return [str(self.target_model.objects.get(**get_kwargs)) or ""]
        return []

    @property
    def media(self) -> Media:
        extra = "" if settings.DEBUG else ".min"
        i18n_name = SELECT2_TRANSLATIONS.get(get_language())
        i18n_file = (f"admin/js/vendor/select2/i18n/{i18n_name}.js",) if i18n_name else ()
        return Media(
            js=(
                f"admin/js/vendor/jquery/jquery{extra}.js",
                f"admin/js/vendor/select2/select2.full{extra}.js",
                *i18n_file,
                "admin/js/jquery.init.js",
                "admin/js/autocomplete.js",
                f"adminfilters/autocomplete{extra}.js",
            ),
            css={
                "screen": (
                    f"admin/css/vendor/select2/select2{extra}.css",
                    self.css % extra,
                ),
            },
        )

    @classmethod
    def factory(cls, *, title: str | None = None, lookup_name: str = "exact", **kwargs: Any) -> type:
        kwargs["filter_title"] = title
        kwargs["lookup_name"] = lookup_name
        return type("ValueFilter", (cls,), kwargs)

    def get_title(self) -> str:
        if not self.can_negate and self.negated:
            if self.negated_title:
                return self.negated_title
            return f"not {self.title}"
        return self.filter_title or self.title


class LinkedAutoCompleteFilter(AutoCompleteFilter):
    parent = None
    parent_lookup_kwarg = None
    extras: ClassVar[list] = []
    dependants: ClassVar[list[str]] = []

    def __init__(
        self,
        field: Field,
        request: HttpRequest,
        params: dict[str, str],
        model: Model,
        model_admin: ModelAdmin,
        field_path: str,
    ) -> None:
        self.dependants.clear()
        if self.parent:
            self.parent_lookup_kwarg = f"{self.parent}__exact"
        super().__init__(field, request, params, model, model_admin, field_path)
        for _pos, entry in enumerate(model_admin.list_filter):
            if isinstance(entry, (list, tuple)) and (
                len(entry) == 2  # noqa: PLR2004
                and entry[0] != self.field_path
                and entry[1].__name__ == type(self).__name__
                and entry[1].parent == self.field_path
            ):
                kwarg = f"{entry[0]}__exact"
                if entry[1].parent and kwarg not in self.dependants:
                    self.dependants.extend(entry[1].dependants)
                    self.dependants.append(kwarg)

    def has_output(self) -> bool:
        if self.parent:
            return self.parent_lookup_kwarg in self.request.GET
        return True

    def choices(self, changelist: ChangeList) -> list[str]:
        to_remove = [self.lookup_kwarg, self.lookup_kwarg_isnull]
        p = changelist.params.copy()
        to_remove.extend(f for f in p if f in self.dependants)

        self.query_string = changelist.get_query_string(remove=to_remove)
        if self.lookup_val:
            return [str(self.target_model.objects.get(pk=self.lookup_val)) or ""]
        return []

    def get_url(self) -> str:
        url = reverse(f"{self.admin_site.name}:autocomplete")
        if self.parent_lookup_kwarg in self.request.GET:
            flt = self.parent_lookup_kwarg.split("__")[-2]
            oid = self.request.GET[self.parent_lookup_kwarg]
            return f"{url}?{flt}={oid}"
        return url

    @classmethod
    def factory(cls, *, title: str | None = None, lookup_name: str = "exact", **kwargs: Any) -> "type":
        kwargs["filter_title"] = title
        kwargs["lookup_name"] = lookup_name
        return type("LinkedAutoCompleteFilter", (cls,), kwargs)



from __future__ import annotations

import logging
import re
from itertools import chain
from typing import TYPE_CHECKING, Any, Callable
from urllib.parse import urlencode

from django.core.exceptions import FieldError, ValidationError
from django.db import models
from django.db.models import BooleanField, DateField, DateTimeField, Field, ForeignObjectRel, Model

if TYPE_CHECKING:
    from django.contrib.contenttypes.fields import GenericForeignKey
    from django.http import HttpRequest


logger = logging.getLogger(__name__)

rex = re.compile(r"'.*'")


def parse_bool(value: Any) -> bool:
    if str(value).lower() in {"true", "1", "yes", "t", "y"}:
        return True
    if str(value).lower() in {"false", "0", "no", "f", "n"}:
        return False
    return value


def get_message_from_exception(e: FieldError) -> str | None:
    message = str(e)
    fieldname = rex.findall(message) or [""]
    if "Unsupported lookup" in message:
        return f"Unsupported lookup: {fieldname[0]}"
    return message


def get_query_string(
    request: HttpRequest, new_params: dict[str, str] | None = None, remove: list[str | None] | None = None
) -> str:
    if new_params is None:
        new_params = {}
    if remove is None:
        remove = []
    p = dict(request.GET.items()).copy()
    for r in remove:
        for k in list(p):
            if k.startswith(r):
                del p[k]
    for k, v in new_params.items():
        if v is None:
            if k in p:
                del p[k]
        else:
            p[k] = v
    return f"?{urlencode(sorted(p.items()))}"


def get_field_by_name(
    model: Model, name: str
) -> tuple[Field | ForeignObjectRel | GenericForeignKey, type[Model], bool, bool]:
    field = model._meta.get_field(name)
    direct = not field.auto_created or field.concrete
    return field, field.model, direct, field.many_to_many


def get_all_field_names(model: Model) -> list[str]:
    if not model:
        raise ValueError("'model' must be a Model instance")
    return list(
        set(
            chain.from_iterable(
                ((field.name, field.attname) if hasattr(field, "attname") else (field.name,))
                for field in model._meta.get_fields()
                if not (field.many_to_one and field.related_model is None)
            )
        )
    )


def get_field_by_path(model: Model, field_path: str) -> Field:
    """
    get a Model class or instance and a path to a attribute, returns the field object

    :param model: :class:`django.db.models.Model`
    :param field_path: string path to the field
    :return: :class:`django.db.models.Field`


    >>> from django.contrib.auth.models import Permission

    >>> p = Permission(name="perm")
    >>> get_field_by_path(Permission, "content_type").name
    'content_type'
    >>> p = Permission(name="perm")
    >>> get_field_by_path(p, "content_type.app_label").name
    'app_label'
    """
    parts = field_path.split(".")
    target = parts[0]
    if not model:
        raise ValueError("'model' must be a Model instance")

    if target in get_all_field_names(model):
        field_object, model, _direct, _m2m = get_field_by_name(model, target)
        if isinstance(field_object, models.ForeignKey):
            if parts[1:]:
                while True:
                    if field_object.related_model is None:
                        break
                    fk = get_field_by_path(field_object.related_model, ".".join(parts[1:]))
                    if fk is None:
                        break
                    field_object = fk
                return field_object
            return field_object
        return field_object
    return None


def get_field_type(model: Model, field_path: str) -> tuple[Field, str, str]:
    lookup = "exact"
    parts = field_path.split("__")
    field = get_field_by_path(model, ".".join(parts))
    if not field:
        raise ValidationError(f"Unknown field '{field_path}'")
    if field.name != parts[-1]:
        lookup = parts[-1]
    field_type = field.get_internal_type()
    return field, lookup, field_type


def cast_value(v: Any, fld: Any, lookup: str, force: Callable[[Any], Any] | None = None) -> Any:
    if force:
        func = force
    elif isinstance(fld, (BooleanField,)) or lookup == "isnull":
        func = parse_bool
    elif isinstance(fld, (DateField, DateTimeField)) and lookup:
        return v
    else:
        func = fld.to_python
    if lookup == "in":
        return [func(e) for e in v.split(",")]
    return func(v)



from typing import Any

from django import forms
from django.contrib.admin import AdminSite, FieldListFilter, ListFilter
from django.contrib.admin.options import ModelAdmin
from django.contrib.admin.views.main import ChangeList
from django.core import checks
from django.core.exceptions import FieldDoesNotExist
from django.db.models import Field, Model, QuerySet
from django.http import HttpRequest

from adminfilters.compat import DJANGO_5


class WrapperMixin:
    negated: bool = False
    can_negate: bool = False
    title: str = ""
    negated_title: str = ""
    placeholder: str = ""

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        self.error = None
        self.error_message = None
        super().__init__(*args, **kwargs)
        if hasattr(self, "media") and self.model_admin and not isinstance(self.model_admin, AdminFiltersMixin):
            raise AttributeError(f"{self.model_admin.__class__.__name__} must inherit from AdminFiltersMixin")

    def get_parameters(
        self, param_name: str, default: str = "", multi: bool = False, pop: bool = False, separator: str = ","
    ) -> str:
        val = self._params.pop(param_name, default) if pop else self._params.get(param_name, default)
        if val:
            if DJANGO_5:
                if isinstance(val, list) and not multi:
                    val = val[-1]
            elif multi:
                val = val.split(separator)
        return val

    def html_attrs(self) -> dict[str, str]:
        classes = f"adminfilters box {self.__class__.__name__.lower()}"
        if self.error_message:
            classes += " error"

        return {
            "class": classes,
            "id": "_".join(self.expected_parameters()),
        }

    def get_title(self) -> str:
        if not self.can_negate and self.negated:
            if self.negated_title:
                return self.negated_title
            return f"not {self.title}"
        return self.title


class SmartListFilter(WrapperMixin, ListFilter):
    def __init__(self, request: HttpRequest, params: dict[str, str], model: Model, model_admin: ModelAdmin) -> None:
        self.model_admin = model_admin
        self._params = params
        super().__init__(request, params, model, model_admin)


class SmartFieldListFilter(WrapperMixin, FieldListFilter):
    def __init__(
        self,
        field: Field,
        request: HttpRequest,
        params: dict[str, str],
        model: Model,
        model_admin: ModelAdmin,
        field_path: str,
    ) -> None:
        self.model_admin = model_admin
        self._params = params.copy()
        super().__init__(field, request, params, model, model_admin, field_path)


class MediaDefinitionFilter:
    pass


class AdminFiltersMixin(ModelAdmin):
    def _check_linked_fields_modeladmin(self) -> list[checks.Error]:  # noqa: C901
        from .autocomplete import LinkedAutoCompleteFilter  # noqa: PLC0415

        linked_filters = [
            e for e in self.list_filter if isinstance(e, (list, tuple)) and issubclass(e[1], LinkedAutoCompleteFilter)
        ]
        errs = []
        seen = []
        for entry in linked_filters:
            if entry[1] and entry[1].parent:
                parts = entry[1].parent.split("__")
                m = self.model
                for part in parts:
                    try:
                        m = m._meta.get_field(part).remote_field.model
                    except FieldDoesNotExist as e:  # noqa: PERF203
                        errs.append(
                            checks.Error(
                                f"{m}` {e}",
                                obj=self,
                                id="adminfilters.E001",
                            )
                        )
                    else:
                        try:
                            ma: ModelAdmin = self.admin_site._registry[m]
                        except KeyError:
                            for proxy in m.__subclasses__():
                                if proxy in self.admin_site._registry:
                                    ma = self.admin_site._registry[proxy]
                                    break
                            else:
                                errs.append(
                                    checks.Error(
                                        f"{m}` is not registered in {self.admin_site}",
                                        obj=self,
                                        id="adminfilters.E002",
                                    )
                                )
                        else:
                            if ma not in seen and not isinstance(ma, AdminAutoCompleteSearchMixin):
                                errs.append(
                                    checks.Error(
                                        f"{ma}` must inherits from AdminAutoCompleteSearchMixin",
                                        obj=ma.__class__,
                                        id="adminfilters.E003",
                                    )
                                )
                                seen.append(ma)

        return errs

    def _check_linked_fields_order(self) -> list[checks.Error]:
        from .autocomplete import LinkedAutoCompleteFilter  # noqa: PLC0415

        linked_filters = [
            e for e in self.list_filter if isinstance(e, (list, tuple)) and issubclass(e[1], LinkedAutoCompleteFilter)
        ]
        errs = []
        seen = []
        for entry in linked_filters:
            if entry[1] and entry[1].parent and entry[1].parent not in seen:
                errs.append(
                    checks.Error(
                        f"Invalid Filters ordering. '{entry[1].parent}' must be defined before '{entry[0]}'.",
                        obj=self.__class__,
                        id="admin.E040",
                    )
                )
            seen.append(entry[0])
        return errs

    def check(self, **kwargs: Any) -> list[checks.Error]:
        return [
            *super().check(**kwargs),
            *self._check_linked_fields_order(),
            *self._check_linked_fields_modeladmin(),
        ]

    def get_changelist_instance(self, request: HttpRequest) -> ChangeList:
        cl = super().get_changelist_instance(request)
        for flt in cl.filter_specs:
            if hasattr(flt, "media"):
                self.admin_filters_media += flt.media
        return cl

    def __init__(self, model: Model, admin_site: AdminSite) -> None:
        self.admin_filters_media = forms.Media()
        super().__init__(model, admin_site)

    @property
    def media(self) -> forms.Media:
        original = super().media
        if hasattr(self, "admin_filters_media"):
            original += self.admin_filters_media
        return original


class AdminAutoCompleteSearchMixin(ModelAdmin):
    def get_search_results(self, request: HttpRequest, queryset: QuerySet, search_term: str) -> tuple[QuerySet, bool]:
        field_names = [f.name for f in self.model._meta.get_fields()]
        filters = {k: v for k, v in request.GET.items() if k in field_names}
        queryset = queryset.filter(**filters)
        queryset, may_have_duplicates = super().get_search_results(request, queryset, search_term)
        return queryset, may_have_duplicates



from __future__ import annotations

from typing import TYPE_CHECKING

from django import forms
from django.conf import settings
from django.contrib.admin import ListFilter
from django.contrib.admin.widgets import SELECT2_TRANSLATIONS
from django.contrib.contenttypes.models import ContentType
from django.utils.translation import get_language

from adminfilters.depot.models import StoredFilter
from adminfilters.mixin import WrapperMixin
from adminfilters.utils import get_query_string

if TYPE_CHECKING:
    from collections.abc import Iterator

    from django.contrib.admin import ModelAdmin
    from django.contrib.admin.filters import _ListFilterChoices
    from django.contrib.admin.views.main import ChangeList
    from django.db.models import Model, QuerySet
    from django.http import HttpRequest


class DepotManager(WrapperMixin, ListFilter):
    title = "Saved Filters"
    template = "adminfilters/widget.html"
    parameter_name = "adminfilters_depot"
    parameter_name_op = "adminfilters_depot_op"

    def __init__(self, request: HttpRequest, params: dict[str, str], model: Model, model_admin: ModelAdmin) -> None:
        self.model_admin = model_admin
        self._params = params.copy()
        super().__init__(request, params, model, model_admin)
        self.request = request
        self.query_string = get_query_string(self.request, {}, self.expected_parameters())
        self.can_add_filter = request.user.has_perm("depot.add_storedfilter")
        self.content_type = ContentType.objects.get_for_model(model_admin.model)
        for p in self.expected_parameters():
            if p in params:
                self.used_parameters[p] = params.pop(p)

    def has_output(self) -> bool:  # noqa: PLR6301
        return True

    def expected_parameters(self) -> list[str | None]:
        return [self.parameter_name, self.parameter_name_op]

    def queryset(self, request: HttpRequest, queryset: QuerySet) -> QuerySet[StoredFilter]:
        filter_name = self.get_parameters(self.parameter_name)
        operation = self.get_parameters(self.parameter_name_op, "add")
        if filter_name:
            if operation == "add":
                qs = get_query_string(request, {}, self.expected_parameters())

                StoredFilter.objects.update_or_create(
                    content_type=self.content_type,
                    name=filter_name,
                    defaults={
                        "query_string": qs,
                        "owner": request.user,
                    },
                )
                self.model_admin.message_user(self.request, f"Filter '{filter_name}' successfully saved")
            elif operation == "delete":
                StoredFilter.objects.filter(id=filter_name).delete()
        return queryset

    def choices(self, changelist: ChangeList) -> "Iterator[_ListFilterChoices]":  # noqa: ARG002
        self.selected = False
        for f in StoredFilter.objects.filter(content_type=self.content_type).order_by("name"):
            if str(self.query_string) == str(f.query_string):
                self.selected = True
                self.selected_id = f.pk

            yield {
                "selected": str(self.query_string) == str(f.query_string),
                "query_string": f.query_string,
                "display": f.name,
            }

    @property
    def media(self) -> forms.Media:
        extra = "" if settings.DEBUG else ".min"
        i18n_name = SELECT2_TRANSLATIONS.get(get_language())
        i18n_file = (f"admin/js/vendor/select2/i18n/{i18n_name}.js",) if i18n_name else ()
        return forms.Media(
            js=(
                f"admin/js/vendor/jquery/jquery{extra}.js",
                *i18n_file,
                "admin/js/jquery.init.js",
                f"adminfilters/depot{extra}.js",
            ),
            css={
                "screen": ("adminfilters/adminfilters.css",),
            },
        )



# Generated by Django 3.2.6 on 2022-02-03 12:38

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="StoredFilter",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=100)),
                ("query_string", models.TextField(blank=True, default="", null=True)),
                (
                    "content_type",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="contenttypes.contenttype",
                    ),
                ),
                (
                    "owner",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
    ]



from .autocomplete import AutoCompleteFilter, LinkedAutoCompleteFilter
from .checkbox import RelatedFieldCheckBoxFilter
from .combo import AllValuesComboFilter, ChoicesFieldComboFilter, RelatedFieldComboFilter
from .dj import DjangoLookupFilter
from .extra import PermissionPrefixFilter
from .json_filter import JsonFieldFilter
from .mixin import AdminAutoCompleteSearchMixin, AdminFiltersMixin
from .multiselect import IntersectionFieldListFilter, UnionFieldListFilter
from .num import MaxMinFilter, NumberFilter
from .querystring import QueryStringFilter
from .radio import (
    AllValuesRadioFilter,
    BooleanRadioFilter,
    ChoicesFieldRadioFilter,
    RelatedFieldRadioFilter,
)
from .value import MultiValueFilter, ValueFilter

__all__ = (
    "AdminAutoCompleteSearchMixin",
    "AdminFiltersMixin",
    "AllValuesComboFilter",
    "AllValuesRadioFilter",
    "AutoCompleteFilter",
    "BooleanRadioFilter",
    "ChoicesFieldComboFilter",
    "ChoicesFieldRadioFilter",
    "DjangoLookupFilter",
    "IntersectionFieldListFilter",
    "JsonFieldFilter",
    "LinkedAutoCompleteFilter",
    "MaxMinFilter",
    "MultiValueFilter",
    "NumberFilter",
    "PermissionPrefixFilter",
    "QueryStringFilter",
    "RelatedFieldCheckBoxFilter",
    "RelatedFieldComboFilter",
    "RelatedFieldRadioFilter",
    "UnionFieldListFilter",
    "ValueFilter",
)



from __future__ import annotations

from typing import TYPE_CHECKING, NoReturn

from django.contrib.admin.utils import prepare_lookup_value
from django.db.models.fields import AutoField, BigAutoField, Field, IntegerField
from django.utils.translation import gettext_lazy as _

from adminfilters.mixin import SmartFieldListFilter

if TYPE_CHECKING:
    from collections.abc import Generator

    from django.contrib.admin import ModelAdmin
    from django.contrib.admin.views.main import ChangeList
    from django.db.models import Model, QuerySet
    from django.http import HttpRequest


class MultipleSelectFieldListFilter(SmartFieldListFilter):
    def __init__(
        self,
        field: Field,
        request: HttpRequest,
        params: dict[str, str],
        model: Model,
        model_admin: ModelAdmin,
        field_path: str,
    ) -> None:
        self.lookup_kwarg = f"{field_path}_filter"
        self.filter_statement = f"{field_path}"

        self._params = params
        self.lookup_val = self.get_parameters(self.lookup_kwarg, pop=True)

        self.lookup_choices = field.get_choices(include_blank=False)
        super().__init__(field, request, params, model, model_admin, field_path)
        self.used_parameters[self.lookup_kwarg] = prepare_lookup_value(self.lookup_kwarg, self.lookup_val)

    def expected_parameters(self) -> list[str | None]:
        return [self.lookup_kwarg]

    def get_field(self) -> Field:
        return self.field.remote_field.model._meta.pk

    def values(self) -> list[str]:
        """
        Returns a list of values to filter on.
        """
        values = []
        value = self.used_parameters.get(self.lookup_kwarg, None)
        if value:
            values = value.split(",")

        field = self.get_field()
        # convert to integers if IntegerField
        if type(field) in {IntegerField, AutoField, BigAutoField}:
            values = [int(x) for x in values]
        return values

    def queryset(self, request: HttpRequest, queryset: QuerySet) -> NoReturn:
        raise NotImplementedError

    def choices(self, cl: ChangeList) -> Generator[dict[str, str | bool], None, None]:
        yield {
            "selected": self.lookup_val is None,
            "query_string": cl.get_query_string({}, [self.lookup_kwarg]),
            "display": _("All"),
        }
        for pk_val, val in self.lookup_choices:
            selected = pk_val in self.values()
            pk_list = set(self.values())
            if selected:
                pk_list.remove(pk_val)
            else:
                pk_list.add(pk_val)
            queryset_value = ",".join([str(x) for x in pk_list])
            if pk_list:
                query_string = cl.get_query_string({
                    self.lookup_kwarg: queryset_value,
                })
            else:
                query_string = cl.get_query_string({}, [self.lookup_kwarg])
            yield {
                "selected": selected,
                "query_string": query_string,
                "display": val,
            }


class IntersectionFieldListFilter(MultipleSelectFieldListFilter):
    """
    A FieldListFilter which allows multiple selection of
    filters for many-to-many type fields. A list of objects will be
    returned whose m2m contains all the selected filters.
    """

    def queryset(self, request: HttpRequest, queryset: QuerySet) -> QuerySet:  # noqa: ARG002
        for value in self.values():
            filter_dct = {self.filter_statement: value}
            queryset = queryset.filter(**filter_dct)
        return queryset


class UnionFieldListFilter(MultipleSelectFieldListFilter):
    """
    A FieldListFilter which allows multiple selection of
    filters for many-to-many type fields, or any type with choices.
    A list of objects will be returned whose m2m or value set
    contains one of the selected filters.
    """

    def get_field(self) -> Field:
        try:
            field = super().get_field()
        except AttributeError as e:  # pragma: no cover
            if hasattr(self.field, "choices") and self.field.choices:
                field = self.field  # It's a *Field with choices
            else:
                raise AttributeError("Multiselect field must be a FK or any type with choices") from e
        return field

    def queryset(self, request: HttpRequest, queryset: QuerySet) -> QuerySet:  # noqa: ARG002
        filter_values = self.values()
        if filter_values:
            filter_statement = f"{self.filter_statement}__in"
            filter_dct = {filter_statement: filter_values}
            queryset = queryset.filter(**filter_dct).distinct()
        return queryset



from __future__ import annotations

import re
from typing import TYPE_CHECKING, Any, ClassVar

from django.contrib.admin.options import IncorrectLookupParameters

from .value import ValueFilter

if TYPE_CHECKING:
    from django.db.models import QuerySet
    from django.http import HttpRequest


class NumberFilter(ValueFilter):
    rex1 = re.compile(r"^(>=|<=|>|<|=)?([-+]?[0-9]+)$")
    re_range = re.compile(r"^(\d+)\.{2,}(\d+)$")
    re_list = re.compile(r"(\d+),?")
    re_unlike = re.compile(r"^(<>)([-+]?[0-9]+)$")
    map: ClassVar[dict[str, str]] = {
        ">=": "gte",
        "<=": "lte",
        ">": "gt",
        "<": "lt",
        "=": "exact",
        "<>": "not",
    }
    can_negate = False

    @classmethod
    def factory(cls, *, title: str | None = None, **kwargs: Any) -> type:
        if "lookup_name" in kwargs:
            raise ValueError(f"'lookup_name' is not a valid value for '{cls.__class__.__name__}.factory'")
        return super().factory(title=title, **kwargs)

    def placeholder(self) -> str:  # noqa: PLR6301
        return "1 or >< <=> <> 1 or 1..10 or 1,4,5"

    def expected_parameters(self) -> list[str | None]:
        self.lookup_kwarg = self.field_path
        return [self.lookup_kwarg]

    def value(self) -> list[str]:
        return [
            self.get_parameters(self.lookup_kwarg),
        ]

    def queryset(self, request: HttpRequest, queryset: QuerySet) -> QuerySet:  # noqa: ARG002
        if self.value() and self.value()[0]:
            raw_value = self.value()[0]
            m1 = self.rex1.match(raw_value)
            m_range = self.re_range.match(raw_value)
            m_list = self.re_list.match(raw_value)
            m_unlike = self.re_unlike.match(raw_value)
            if m_unlike and m_unlike.groups():
                match = f"{self.field_path}__exact"
                op, value = self.re_unlike.match(raw_value).groups()
                queryset = queryset.exclude(**{match: value})
            else:
                if m1 and m1.groups():
                    op, value = self.rex1.match(raw_value).groups()
                    match = "{}__{}".format(self.field_path, self.map[op or "="])
                    self.filters = {match: value}
                elif m_range and m_range.groups():
                    start, end = self.re_range.match(raw_value).groups()
                    self.filters = {
                        f"{self.field_path}__gte": start,
                        f"{self.field_path}__lte": end,
                    }
                elif m_list and m_list.groups():
                    value = raw_value.split(",")
                    match = f"{self.field_path}__in"
                    self.filters = {match: value}
                else:  # pragma: no cover
                    raise IncorrectLookupParameters
                try:
                    queryset = queryset.filter(**self.filters)
                except Exception as e:
                    raise IncorrectLookupParameters(self.value()) from e
        return queryset


# backward compatibility
MaxMinFilter = NumberFilter



from __future__ import annotations

import logging
from typing import TYPE_CHECKING, Any
from urllib import parse

from django import forms
from django.conf import settings
from django.contrib.admin.widgets import SELECT2_TRANSLATIONS
from django.core.exceptions import FieldError, ValidationError
from django.utils.translation import get_language
from django.utils.translation import gettext as _

from .mixin import MediaDefinitionFilter, SmartListFilter
from .utils import cast_value, get_field_type, get_message_from_exception

if TYPE_CHECKING:
    from django.contrib.admin import ModelAdmin
    from django.contrib.admin.views.main import ChangeList
    from django.db.models import Model, QuerySet
    from django.http import HttpRequest

logger = logging.getLogger(__name__)


class QueryStringFilter(MediaDefinitionFilter, SmartListFilter):
    parameter_name = "qs"
    title = "QueryString"
    template = "adminfilters/querystring.html"
    can_negate = True
    negated = False
    options = True
    separator = ","
    placeholder = _("django db lookup values")

    def __init__(self, request: HttpRequest, params: dict[str, str], model: Model, model_admin: ModelAdmin) -> None:
        self.parameter_name_negated = f"{self.parameter_name}__negate"
        self._params = params
        self.lookup_field_val = self.get_parameters(self.parameter_name, pop=True)
        self.lookup_negated_val = self.get_parameters(self.parameter_name_negated, "false", pop=True)
        self.query_string = None
        self.error_message = None
        self.exception = None
        self.filters = None
        self.exclude = None
        self.model_admin = model_admin
        self.model = model
        self.validation_errors = {}
        self.filters = {}
        self.exclude = {}
        super().__init__(request, params, model, model_admin)

    @classmethod
    def factory(cls, **kwargs: Any) -> type:
        return type("QueryStringFilter", (cls,), kwargs)

    def expected_parameters(self) -> list[str | None]:
        return [self.parameter_name, self.parameter_name_negated]

    def has_output(self) -> bool:  # noqa: PLR6301
        return True

    def value(self) -> list[str | bool]:
        return [self.lookup_field_val, (self.can_negate and self.lookup_negated_val == "true") or self.negated]

    def choices(self, changelist: ChangeList) -> list[str]:
        self.query_string = changelist.get_query_string(remove=self.expected_parameters())
        return []

    def get_filters(self, value: Any) -> tuple[dict[str, str], dict[str, str]]:
        query_params = dict(parse.parse_qsl("&".join(value.splitlines())))
        exclude = {}
        filters = {}
        for fname, raw_value in query_params.items():
            target = filters
            cast = None
            field_name = fname

            if field_name[0] == "!":
                field_name = field_name[1:]
                target = exclude

            if field_name[0] == "#":
                field_name = field_name[1:]
                cast = int
            if field_name[0] == ".":
                field_name = field_name[1:]
                cast = float

            field, lookup, _field_type = get_field_type(self.model, field_name)
            value = cast_value(raw_value, field, lookup, force=cast)
            target[field_name] = value

        return filters, exclude

    def queryset(self, request: HttpRequest, queryset: QuerySet) -> QuerySet:  # noqa: ARG002
        value, negated = self.value()
        if value:
            try:
                self.filters, self.exclude = self.get_filters(value)
                if not (self.filters or self.exclude):
                    self.error_message = _("Invalid django filter")
                elif negated:
                    queryset = queryset.filter(**self.exclude).exclude(**self.filters)
                else:
                    queryset = queryset.filter(**self.filters).exclude(**self.exclude)
            except FieldError as e:
                self.error_message = get_message_from_exception(e)
            except ValidationError as e:  # pragma: no cover
                self.error_message = ", ".join(e.messages)
            except Exception as e:  # pragma: no cover
                logger.exception(e)
                if settings.DEBUG:
                    self.error_message = f"{e.__class__.__name__}: {e}"
                else:
                    logger.exception(e)
                    self.error_message = "Invalid filter"
        return queryset

    @property
    def media(self) -> forms.Media:
        extra = "" if settings.DEBUG else ".min"
        i18n_name = SELECT2_TRANSLATIONS.get(get_language())
        i18n_file = (f"admin/js/vendor/select2/i18n/{i18n_name}.js",) if i18n_name else ()
        return forms.Media(
            js=(
                f"admin/js/vendor/jquery/jquery{extra}.js",
                *i18n_file,
                "admin/js/jquery.init.js",
                f"adminfilters/querystring{extra}.js",
            ),
            css={
                "screen": ("adminfilters/adminfilters.css",),
            },
        )



from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.db import models


class StoredFilter(models.Model):
    name = models.CharField(blank=False, null=False, max_length=100)
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    query_string = models.TextField(blank=True, default="")
    owner = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)

    class Meta:
        ordering = ("name",)

    def __str__(self) -> str:
        return self.name



from django.contrib import admin
from django.contrib.admin import register

from adminfilters.combo import RelatedFieldComboFilter

from .models import StoredFilter


@register(StoredFilter)
class StoredFilterAdmin(admin.ModelAdmin):
    search_fields = ("name",)
    list_display = ("name", "owner", "content_type")
    raw_id_fields = ("owner", "content_type")
    list_filter = (("content_type", RelatedFieldComboFilter),)



from django.contrib.admin.filters import AllValuesFieldListFilter, ChoicesFieldListFilter, RelatedFieldListFilter

from .mixin import WrapperMixin


class AllValuesComboFilter(WrapperMixin, AllValuesFieldListFilter):
    template = "adminfilters/combobox.html"


class RelatedFieldComboFilter(WrapperMixin, RelatedFieldListFilter):
    template = "adminfilters/combobox.html"


class ChoicesFieldComboFilter(WrapperMixin, ChoicesFieldListFilter):
    template = "adminfilters/combobox.html"



from __future__ import annotations

import json
from typing import TYPE_CHECKING, Any

from django import forms
from django.conf import settings
from django.contrib import messages
from django.contrib.admin.widgets import SELECT2_TRANSLATIONS
from django.utils.translation import get_language
from django.utils.translation import gettext as _

from adminfilters.mixin import MediaDefinitionFilter, SmartFieldListFilter

if TYPE_CHECKING:
    from django.contrib.admin import ModelAdmin
    from django.contrib.admin.views.main import ChangeList
    from django.db.models import Field, Model, QuerySet
    from django.http import HttpRequest


class ValueFilter(MediaDefinitionFilter, SmartFieldListFilter):
    template = "adminfilters/value.html"
    toggleable = False
    filter_title = None
    lookup_name = "exact"
    button = True
    can_negate = True
    negated = False

    def __init__(
        self,
        field: Field,
        request: HttpRequest,
        params: dict[str, str],
        model: Model,
        model_admin: ModelAdmin,
        field_path: str,
    ) -> None:
        self.lookup_kwarg = None
        self.lookup_kwarg_negated = None
        self.field_path = field_path
        self.parameters = {}
        self.filters = {}
        for p in self.expected_parameters():
            if p in params:
                self.parameters[p] = params.pop(p)

        super().__init__(field, request, params, model, model_admin, field_path)
        self._params = self.parameters
        self.title = self._get_title()

    def expected_parameters(self) -> list[str | None]:
        self.lookup_kwarg = f"{self.field_path}__{self.lookup_name}"
        self.lookup_kwarg_negated = f"{self.lookup_kwarg}__negate"
        return [self.lookup_kwarg, self.lookup_kwarg_negated]

    def value(self) -> tuple[str, bool]:
        return (
            self.get_parameters(self.lookup_kwarg),
            self.get_parameters(self.lookup_kwarg_negated) == "true",
        )

    def js_options(self) -> str:
        return json.dumps({"button": self.button, "canNegate": self.can_negate, "negated": self.negated})

    def _get_title(self) -> str:
        if self.filter_title:
            return self.filter_title
        if "__" in self.field_path:
            return self.field_path.replace("__", "->")
        return getattr(self.field, "verbose_name", self.field_path)

    @classmethod
    def factory(cls, *, title: str | None = None, lookup_name: str = "exact", **kwargs: Any) -> type:
        kwargs["filter_title"] = title
        kwargs["lookup_name"] = lookup_name
        return type("ValueFilter", (cls,), kwargs)

    def queryset(self, request: HttpRequest, queryset: QuerySet) -> QuerySet[Model]:
        target, exclude = self.value()
        if target:
            try:
                self.filters = {self.lookup_kwarg: target}
                queryset = queryset.exclude(**self.filters) if exclude else queryset.filter(**self.filters)
                """
                if exclude:
                    queryset = queryset.exclude(**self.filters)
                else:
                    queryset = queryset.filter(**self.filters)
                """
            except Exception as e:  # noqa: BLE001
                msg = _("%s filter ignored due to an error %s") % (self.title, e)
                self.model_admin.message_user(request, msg, messages.ERROR)
        return queryset

    def choices(self, changelist: ChangeList) -> list:
        self.query_string = changelist.get_query_string(remove=self.expected_parameters())
        return []

    @property
    def media(self) -> forms.Media:
        extra = "" if settings.DEBUG else ".min"
        i18n_name = SELECT2_TRANSLATIONS.get(get_language())
        i18n_file = (f"admin/js/vendor/select2/i18n/{i18n_name}.js",) if i18n_name else ()
        return forms.Media(
            js=(
                f"admin/js/vendor/jquery/jquery{extra}.js",
                *i18n_file,
                "admin/js/jquery.init.js",
                f"adminfilters/value{extra}.js",
            ),
            css={
                "screen": ("adminfilters/adminfilters.css",),
            },
        )


class MultiValueFilter(ValueFilter):
    template: str = "adminfilters/value_multi.html"
    separator: str = ","
    filter_title: str = ""
    lookup_name: str = "in"

    def placeholder(self) -> str:  # noqa: PLR6301
        return _("comma separated list of values")

    def value(self) -> list[str]:
        values = self.get_parameters(self.lookup_kwarg, None, multi=True)
        return [values, self.get_parameters(self.lookup_kwarg_negated, "") == "true"]


TextFieldFilter = ValueFilter
ForeignKeyFieldFilter = TextFieldFilter
MultiValueTextFieldFilter = MultiValueFilter



# Generated by Django 4.0.2 on 2023-05-25 10:26

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("depot", "0002_migration"),
    ]

    operations = [
        migrations.AlterField(
            model_name="storedfilter",
            name="id",
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
        ),
    ]



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from django import forms
from django.conf import settings
from django.contrib.admin.widgets import SELECT2_TRANSLATIONS
from django.core.exceptions import FieldError, ValidationError
from django.db.models import Model, Q, QuerySet
from django.utils.translation import get_language
from django.utils.translation import gettext as _

from .mixin import MediaDefinitionFilter, SmartListFilter
from .utils import cast_value, get_field_type, get_message_from_exception

if TYPE_CHECKING:
    from django.contrib.admin import ModelAdmin
    from django.contrib.admin.views.main import ChangeList
    from django.http import HttpRequest


class DjangoLookupFilter(MediaDefinitionFilter, SmartListFilter):
    parameter_name = "dj"
    title = "Django Lookup"
    template = "adminfilters/dj.html"
    can_negate = True
    negated = False
    button = True
    field_placeholder = _("field lookup. Es. name__startswith")
    placeholder = _("field value")

    def __init__(self, request: HttpRequest, params: dict[str, str], model: Model, model_admin: ModelAdmin) -> None:
        self.lookup_kwarg_key = f"{self.parameter_name}__key"
        self.lookup_kwarg_value = f"{self.parameter_name}__value"
        self.lookup_kwarg_negated = f"{self.parameter_name}__negate"
        self._params = params
        self.lookup_field_val = self.get_parameters(self.lookup_kwarg_key, pop=True)
        self.lookup_value_val = self.get_parameters(self.lookup_kwarg_value, pop=True)
        self.lookup_negated_val = self.get_parameters(self.lookup_kwarg_negated, "false", pop=True)
        self.error_message = None
        self.exception = None
        self.filters = None
        self.exclude = None
        self.model = model
        self.query_string = None
        super().__init__(request, params, model, model_admin)

    @classmethod
    def factory(cls, **kwargs: Any) -> type:
        return type("DjangoLookupFilter", (cls,), kwargs)

    def expected_parameters(self) -> list[str | None]:
        return [
            self.lookup_kwarg_key,
            self.lookup_kwarg_value,
            self.lookup_kwarg_negated,
        ]

    def has_output(self) -> bool:  # noqa: PLR6301
        return True

    def value(self) -> Any:
        try:
            if self.lookup_field_val:
                field, lookup, _field_type = get_field_type(self.model, self.lookup_field_val)
                value = cast_value(self.lookup_value_val, field, lookup)
                return [
                    self.lookup_field_val,
                    value,
                    (self.can_negate and self.lookup_negated_val == "true") or self.negated,
                ]
        except ValidationError as e:  # pragma: no cover
            self.exception = e
            self.error_message = str(e.message)

        return [None, None, None]

    def choices(self, changelist: ChangeList) -> list:
        self.query_string = changelist.get_query_string(remove=self.expected_parameters())
        return []

    def queryset(self, request: HttpRequest, queryset: QuerySet) -> QuerySet:  # noqa: ARG002
        try:
            key, value, negated = self.value()
            if key:
                try:
                    self.filters = Q(**{f"{self.lookup_field_val}": value})

                    if negated:
                        self.filters = ~self.filters

                    queryset = queryset.filter(self.filters)
                except FieldError as e:
                    self.exception = e
                    self.error_message = get_message_from_exception(e)
        except ValidationError as e:  # pragma: no cover
            self.exception = e
            self.error_message = str(e.message)

        return queryset

    @property
    def media(self) -> forms.Media:
        extra = "" if settings.DEBUG else ".min"
        i18n_name = SELECT2_TRANSLATIONS.get(get_language())
        i18n_file = (f"admin/js/vendor/select2/i18n/{i18n_name}.js",) if i18n_name else ()
        return forms.Media(
            js=(
                f"admin/js/vendor/jquery/jquery{extra}.js",
                *i18n_file,
                "admin/js/jquery.init.js",
                f"adminfilters/dj{extra}.js",
            ),
            css={
                "screen": ("adminfilters/adminfilters.css",),
            },
        )



from collections.abc import Generator
from typing import Any

from django.contrib.admin import ModelAdmin
from django.contrib.admin.filters import RelatedFieldListFilter
from django.contrib.admin.views.main import ChangeList
from django.db.models import Model, QuerySet
from django.db.models.fields.related import ForeignObjectRel, RelatedField
from django.db.models.query_utils import Q
from django.http import HttpRequest
from django.utils.encoding import smart_str
from django.utils.translation import gettext as _

from .mixin import MediaDefinitionFilter, WrapperMixin
from .utils import parse_bool


class RelatedFieldCheckBoxFilter(WrapperMixin, MediaDefinitionFilter, RelatedFieldListFilter):
    template = "adminfilters/checkbox.html"

    def __init__(
        self,
        field: RelatedField,
        request: HttpRequest,
        params: dict[str, Any],
        model: Model,
        model_admin: ModelAdmin,
        field_path: str,
    ) -> None:
        self.model_admin = model_admin
        super().__init__(field, request, params, model, model_admin, field_path)
        self.lookup_kwarg = f"{field_path}__{field.target_field.name}"
        self.lookup_val = request.GET.getlist(self.lookup_kwarg, [])

    def queryset(self, request: HttpRequest, queryset: QuerySet) -> QuerySet:  # noqa: ARG002
        filters = Q()
        if self.lookup_val:
            filters.add(Q(**{f"{self.lookup_kwarg}__in": self.lookup_val}), Q.OR)

        if self.lookup_val_isnull:
            filters.add(
                Q(**{self.lookup_kwarg_isnull: parse_bool(self.lookup_val_isnull)}),
                Q.OR,
            )
        return queryset.filter(filters)

    def choices(self, cl: ChangeList) -> Generator[dict, None, None]:
        """
        # try:
        #     from django.contrib.admin.views.main import EMPTY_CHANGELIST_VALUE
        # except ImportError:
        """
        EMPTY_CHANGELIST_VALUE = self.model_admin.get_empty_value_display()

        uncheck_all = []
        uncheck_all.append(f"{self.lookup_kwarg_isnull}={1}")
        uncheck_all.extend(f"{self.lookup_kwarg}={i[0]}" for i in self.lookup_choices)

        yield {
            "selected": not len(self.lookup_val) and not self.lookup_val_isnull,
            "query_string": cl.get_query_string({}, [self.lookup_kwarg, self.lookup_kwarg_isnull]),
            "display": _("All"),
            "check_to_remove": "&".join(uncheck_all),
        }
        yield {
            "selected": self.lookup_val_isnull,
            "query_string": cl.get_query_string(
                {self.lookup_kwarg_isnull: 1},
                [self.lookup_kwarg, self.lookup_kwarg_isnull],
            ),
            "display": _("None"),
            "uncheck_to_remove": f"{self.lookup_kwarg_isnull}=1",
        }
        for pk_val, val in self.lookup_choices:
            yield {
                "selected": smart_str(pk_val) in self.lookup_val,
                "query_string": cl.get_query_string(
                    {
                        self.lookup_kwarg: pk_val,
                    },
                    [self.lookup_kwarg_isnull],
                ),
                "display": val,
                "uncheck_to_remove": (f"{self.lookup_kwarg}={pk_val}" if pk_val else ""),
            }
        if (isinstance(self.field, ForeignObjectRel) and self.field.field.null) or (
            hasattr(self.field, "rel") and self.field.null
        ):
            yield {
                "selected": bool(self.lookup_val_isnull),
                "query_string": cl.get_query_string(
                    {
                        self.lookup_kwarg_isnull: "True",
                    },
                    [self.lookup_kwarg],
                ),
                "uncheck_to_remove": f"{self.lookup_kwarg_isnull}=1",
                "display": EMPTY_CHANGELIST_VALUE,
            }



# Generated by Django 3.2.12 on 2022-04-05 21:42

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("depot", "0001_initial"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="storedfilter",
            options={"ordering": ("name",)},
        ),
        migrations.AlterField(
            model_name="storedfilter",
            name="id",
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
        ),
    ]



import re

from django.contrib.admin.options import IncorrectLookupParameters, ModelAdmin
from django.db.models import DateField, DateTimeField, Model, QuerySet
from django.db.models.fields import Field
from django.http import HttpRequest

from .num import NumberFilter


class DateRangeFilter(NumberFilter):
    rex1 = re.compile(r"^(>=|<=|>|<|=)?(\d{4}-\d{2}-\d{2})$")
    re_range = re.compile(r"^(\d{4}-\d{2}-\d{2})..(\d{4}-\d{2}-\d{2})$")
    re_list = re.compile(r"(\d{4}-\d{2}-\d{2}),?")
    re_unlike = re.compile(r"^(<>)(?P<date>\d{4}-\d{2}-\d{2})$")

    def __init__(
        self,
        field: Field,
        request: HttpRequest,
        params: dict[str, str],
        model: Model,
        model_admin: ModelAdmin,
        field_path: str,
    ) -> None:
        super().__init__(field, request, params, model, model_admin, field_path)
        if isinstance(field, DateTimeField):
            self.extra_lookup = "__date"
        elif isinstance(field, DateField):
            self.extra_lookup = ""

    def queryset(self, request: HttpRequest, queryset: QuerySet) -> QuerySet[Model]:  # noqa: ARG002
        if self.value() and self.value()[0]:
            raw_value = self.value()[0]
            m1 = self.rex1.match(raw_value)
            m_range = self.re_range.match(raw_value)
            m_list = self.re_list.match(raw_value)
            m_unlike = self.re_unlike.match(raw_value)
            lk = f"{self.field.name}{self.extra_lookup}"
            if m_unlike and m_unlike.groups():
                match = f"{lk}__exact"
                op, value = self.re_unlike.match(raw_value).groups()
                queryset = queryset.exclude(**{match: value})
            else:
                if m1 and m1.groups():
                    op, value = self.rex1.match(raw_value).groups()
                    match = f"{lk}__{self.map[op or '=']}"
                    self.filters = {match: value}
                elif m_range and m_range.groups():
                    start, end = self.re_range.match(raw_value).groups()
                    self.filters = {f"{lk}__gte": start, f"{lk}__lte": end}
                elif m_list and m_list.groups():
                    value = raw_value.split(",")
                    match = f"{lk}__in"
                    self.filters = {match: value}
                else:  # pragma: no cover
                    raise IncorrectLookupParameters

                try:
                    queryset = queryset.filter(**self.filters)
                except Exception as e:
                    raise IncorrectLookupParameters(self.value()) from e
        return queryset



from collections.abc import Iterable
from typing import Any

from django import forms
from django.conf import settings
from django.contrib.admin import ModelAdmin
from django.contrib.admin.views.main import ChangeList
from django.contrib.admin.widgets import SELECT2_TRANSLATIONS
from django.db.models import Field, Model, Q, QuerySet
from django.forms import Media
from django.http import HttpRequest
from django.utils.translation import get_language
from django.utils.translation import gettext as _

from .mixin import MediaDefinitionFilter, SmartFieldListFilter


class JsonFieldFilter(MediaDefinitionFilter, SmartFieldListFilter):
    parameter_name = None
    title = None
    template = "adminfilters/json.html"
    can_negate = True
    negated = False
    options = True
    key_placeholder = _("JSON key")
    placeholder = _("JSON value")

    def __init__(
        self,
        field: Field,
        request: HttpRequest,
        params: dict[str, str],
        model: Model,
        model_admin: ModelAdmin,
        field_path: str,
    ) -> None:
        self.lookup_kwarg_key = f"{field_path}__key"
        self.lookup_kwarg_value = f"{field_path}__value"
        self.lookup_kwarg_negated = f"{field_path}__negate"
        self.lookup_kwarg_options = f"{field_path}__options"
        self.lookup_kwarg_type = f"{field_path}__type"

        self.field = field
        self.query_string = None
        self.field_path = field_path
        self.title = getattr(field, "verbose_name", field_path)
        super().__init__(field, request, params, model, model_admin, field_path)

        self.lookup_key_val = self.get_parameters(self.lookup_kwarg_key, "")
        self.lookup_value_val = self.get_parameters(self.lookup_kwarg_value, "")
        self.lookup_negated_val = self.get_parameters(self.lookup_kwarg_negated, "false")
        self.lookup_type_val = self.get_parameters(self.lookup_kwarg_type, "any")
        self.lookup_options_val = self.get_parameters(self.lookup_kwarg_options, "e")

    @classmethod
    def factory(cls, **kwargs: Any) -> "type":
        return type("JsonFieldFilter", (cls,), kwargs)

    def expected_parameters(self) -> Iterable[str]:
        return [
            self.lookup_kwarg_key,
            self.lookup_kwarg_value,
            self.lookup_kwarg_negated,
            self.lookup_kwarg_options,
            self.lookup_kwarg_type,
        ]

    def value(self) -> list[str]:
        return [
            self.lookup_key_val,
            self.lookup_value_val,
            self.lookup_options_val,
            (self.can_negate and self.lookup_negated_val == "true") or self.negated,
            self.lookup_type_val,
        ]

    def choices(self, changelist: ChangeList) -> list:
        self.query_string = changelist.get_query_string(remove=self.expected_parameters())
        return []

    def queryset(self, request: HttpRequest, queryset: QuerySet) -> QuerySet:  # noqa: ARG002
        key, value, options, negated, type_ = self.value()
        if key:
            if type_ == "any" and value.isnumeric():
                filters = Q(**{f"{self.field_path}__{key}": value}) | Q(**{f"{self.field_path}__{key}": int(value)})
            elif type_ == "num" and value.isnumeric():
                filters = Q(**{f"{self.field_path}__{key}": float(value)})
            else:  # type_ == 'str':
                filters = Q(**{f"{self.field_path}__{key}": str(value)})

            if negated:
                if self.options and options == "e":
                    filters = ~filters
                else:
                    filters = Q(**{f"{self.field_path}__{key}__isnull": True}) | ~filters
            elif options == "i":
                filters |= Q(**{f"{self.field_path}__{key}__isnull": True})

            queryset = queryset.filter(filters)
        return queryset

    @property
    def media(self) -> Media:
        extra = "" if settings.DEBUG else ".min"
        i18n_name = SELECT2_TRANSLATIONS.get(get_language())
        i18n_file = (f"admin/js/vendor/select2/i18n/{i18n_name}.js",) if i18n_name else ()
        return forms.Media(
            js=(
                f"admin/js/vendor/jquery/jquery{extra}.js",
                *i18n_file,
                "admin/js/jquery.init.js",
                f"adminfilters/jsonfilter{extra}.js",
            ),
            css={
                "screen": ("adminfilters/adminfilters.css",),
            },
        )



from django.contrib.admin.filters import (
    AllValuesFieldListFilter,
    BooleanFieldListFilter,
    ChoicesFieldListFilter,
    RelatedFieldListFilter,
)

from .mixin import WrapperMixin


class AllValuesRadioFilter(WrapperMixin, AllValuesFieldListFilter):
    template = "adminfilters/radio.html"


class RelatedFieldRadioFilter(WrapperMixin, RelatedFieldListFilter):
    template = "adminfilters/radio.html"


class ChoicesFieldRadioFilter(WrapperMixin, ChoicesFieldListFilter):
    template = "adminfilters/radio.html"


class BooleanRadioFilter(WrapperMixin, BooleanFieldListFilter):
    template = "adminfilters/radio.html"



from inspect import isclass
from typing import Any

from django.contrib.admin import ListFilter
from django.core.checks import Error, register

from adminfilters.mixin import AdminFiltersMixin, MediaDefinitionFilter


@register()
def check_adminfilters_media(*args: Any, **kwargs: Any) -> list[Error]:  # noqa: ARG001
    errors = []
    from django.contrib.admin import site  # noqa: PLC0415

    for model_admin in site._registry.values():
        if isclass(model_admin) and not issubclass(model_admin, AdminFiltersMixin):
            for filter_fields in model_admin.list_filter:
                filter_class = None
                if isinstance(filter_fields, tuple):
                    filter_class = filter_fields[1]
                elif isclass(filter_fields) and issubclass(filter_fields, (ListFilter,)):
                    filter_class = filter_fields

                if filter_class and issubclass(filter_class, MediaDefinitionFilter):
                    errors.append(
                        Error(
                            f"To use '{filter_class.__name__}', "
                            f"'{model_admin.__name__}' must inherit from 'AdminFiltersMixin'"
                        )
                    )
                    break

    return errors



from typing import ClassVar

from django.contrib.admin import ModelAdmin, SimpleListFilter
from django.db.models import Q, QuerySet
from django.http import HttpRequest
from django.utils.translation import gettext as _


class PermissionPrefixFilter(SimpleListFilter):
    title = "Permission"
    parameter_name = "perm"
    prefixes: ClassVar[list[tuple[str, str]]] = [
        ("view", _("View")),
        ("add", _("Add")),
        ("change", _("Change")),
        ("delete", _("Delete")),
        ("--", _("Others")),
    ]
    lookup_val = None

    def lookups(self, request: HttpRequest, model_admin: ModelAdmin) -> list[tuple[str, str]]:  # noqa: ARG002
        return self.prefixes

    def queryset(self, request: HttpRequest, queryset: QuerySet) -> QuerySet:  # noqa: ARG002
        if not self.value():
            return queryset
        if self.value() == "--":
            k = [prefix for prefix, label in self.prefixes]
            query = Q()
            for el in k:
                query |= Q(codename__startswith=el)
            return queryset.exclude(query)
        return queryset.filter(codename__startswith=self.value())
